#!/usr/bin/env python
from distutils.core import setup

setup(
    name='django-multifilefield',
    version='1.0',
    description='Django model and form fields for storring html5 forms multiupload data',
    author='Sergey Maltsev',
    author_email='alternativshik@gmail.com',
    url='https://github.com/Cyberic/django-multifilefield',
    packages=[
        'multifilefield',
    ]
)
